# A python script to approximate the function of sudanConference.xml
# Takes input of the format: python sudanConf.py <conference topic id> <attendee 1> <attendee 2> [<attendee 3> <attendee 4>]
# conference with 2 attendees: 4 facts from the given belief
# conference with 3 attendees: 3 facts from the given belief
# conference with 4 attendees: 2 facts from the given belief

#Topic key:
# 0: Belief in the unity of Sudan
# 1: Belief in the uniformity of Sharia across Sudan
# 2: Belief in Human Rights (as approximately defined by the U.N.)
# 3: Belief in the rights of women (as approximately accepted in the U.S.)
# 4: Belief in the National Congress Party's Platform
# 5: Belief in the Sudan Peoples' Liberation Movement Platform
# 6: Belief in the possibility of a peaceful resolution of tribal disputes via political means
# 7: Belief in the possibility of a peaceful resolution of border disputes via political means

#Attendee key:
#  6: Acholi Leader
#  7: Anuak Leader
#  8: Bari Leader
#  9: Bor Dinka Leader
# 10: Gok Dinka Leader
# 11: Malual Dinka Leader
# 12: Ngok Dinka Leader
# 13: Madi Leader
# 14: Mondari Leader
# 15: Murle Leader
# 16: Nuer Leader
# 17: Misseriya Leader
# 18: Rizeigat Leader
# 19: Seleim Leader

import random,sys

INFILE  = 'knowledge_network.csv'
OUTFILE = 'knowledge_network.csv'

FACTS_PER_BELIEF = 10#8
FIRST_PROMOTER = 390

#returns a list of the things in a that are not in b
def listOneWayExclusion(a,b): return [val for val in a if val not in b]

def listToFactList(ls):
	ls.sort()
	ls += [FACTS_PER_BELIEF+1]
	str = ''
	for i in range(FACTS_PER_BELIEF):
		if i == ls[0]:
			str += '1,'
			ls.pop(0)
		else: str += '0,'
	print str[:-1]
	return str[:-1]

def checkVals(argv):
	if len(argv) < 3 or len(argv) > 5:
		print 'This script needs 3 to 5 arguments: python sudanConf.py <conference topic id> <attendee 1> <attendee 2> [<attendee 3> <attendee 4>]'
		sys.exit(2)
	try:
		if int(argv[0]) < 0 or int(argv[0]) > 7:
			print 'The first argument, "'+argv[0]+'", needs to be in the range [0,7].'
			sys.exit(2)			
	except ValueError:
		print 'The first argument, "'+argv[0]+'", needs to be a value that can be parsed as an integer in the range [0,7].'
		sys.exit(2)		
	for arg in argv[1:]:
		try:
			if int(arg) < 6 or int(arg) > 19:
				print '"'+str(arg)+'" needs to be in the range [6,19].'
				sys.exit(2)
		except ValueError:
			print '"'+arg+'" needs to be a value that can be parsed as an integer in the range [6,19].'
			sys.exit(2)
	
def main(argv):
	#check that you have the right number & types of arguments
	checkVals(argv)
	
	attendees = argv[1:]
	for i in range(len(attendees)): attendees[i] = int(attendees[i])+FIRST_PROMOTER
	attendees.sort()
	print 'attendees: '+str(attendees)
	factRange = [int(argv[0])*FACTS_PER_BELIEF,(int(argv[0])+1)*FACTS_PER_BELIEF]
	print 'factRange: '+str(factRange)
	factsToExchangeCount = 6-len(attendees)
	
	
	infile = open(INFILE,'r')
	lines = infile.readlines()
	infile.close()

	#makes up a list of agents, where each agent is represented by the facts that they know: [[1,2,3],[2,5,6,7]...]
	agents = []
	for attendee in attendees:
		lines[attendee] = lines[attendee].strip()
		ls = lines[attendee].split(',')[factRange[0]:factRange[1]]
		print 'ls: '+str(ls)
		
		agent = []
		for i in range(len(ls)):
			if int(ls[i]) == 1: agent += [i]
		
		agents += [agent]
	
	agentsOriginal = agents[:]
	
	for i in range(len(agents)):
		for j in range(i)+range(i+1,len(agents)):
			
			# get the facts that the current agent will teach from its original knowledge base
			# if the agent knows more facts than are to be exchanged, choose a random subset
			# Teach those facts to agent[j]
			
			factsToExchange = agentsOriginal[i]
			if len(factsToExchange) > factsToExchangeCount:
				random.shuffle(factsToExchange)
				factsToExchange = factsToExchange[:factsToExchangeCount]
			
			print factsToExchange
			agents[j] = agents[j]+listOneWayExclusion(factsToExchange,agents[j])
	
	# write out the new knowledge values for the attendees.
	outfile = open(OUTFILE,'w')
	for line in lines[:attendees[0]]:
		outfile.write(line)
		
	start = str(lines[attendees[0]].strip().split(',')[:factRange[0]])[1:-1].replace(' ','').replace('\'','')
	end   = str(lines[attendees[0]].strip().split(',')[factRange[1]:])[1:-1].replace(' ','').replace('\'','')
	
	if start != '': outfile.write(start+',')
	outfile.write(listToFactList(agents[0]))
	if end != '': outfile.write(','+end)
	outfile.write('\n')
	
	for i in range(1,len(attendees)):
		#write gap to this attendee
		for line in lines[(attendees[i-1]+1):attendees[i]]:
			outfile.write(line)
		#write attendee knowledge
		start = str(lines[attendees[i]].strip().split(',')[:factRange[0]])[1:-1].replace(' ','').replace('\'','')
		end   = str(lines[attendees[i]].strip().split(',')[factRange[1]:])[1:-1].replace(' ','').replace('\'','')
		if start != '': outfile.write(start+',')
		outfile.write(listToFactList(agents[i]))
		if end != '': outfile.write(','+end)
		outfile.write('\n')

	for line in lines[(attendees[-1]+1):]:
		outfile.write(line)
		
	outfile.close()
	
if __name__ == "__main__":
	main(sys.argv[1:])